package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.setup.DataSourceTestConfiguration;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {DataSourceTestConfiguration.class})
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
public class BlurringDaoTest {
    @Autowired
    DataSource dataSource;

    private BlurringDao blurringDao;

    private static final BlurringEntity BLURRING1 = new BlurringEntity("1111111111", "cpr", "1111", "cvr", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING2 = new BlurringEntity("1111111111", "something else", "2222", "cvr", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING3 = new BlurringEntity("2222222222", "cpr", "3333", "cvr", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING4 = new BlurringEntity("2222222222", "cpr", "4444", "cvr", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING5 = new BlurringEntity("3333333333", "cpr", "5555", "cvr", instantFrom(2000, 1, 1, 0, 0));

    @Before
    public void setUp() throws Exception {
        blurringDao = new BlurringDao(dataSource);
    }

    @Test
    public void getActiveBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("1111111111", "cpr");

        assertEquals(1, blurrings.size());
        assertEquals(BLURRING1, blurrings.get(0));
    }

    @Test
    public void getActiveBlurrings_WhenTwoMatchingActiveBlurringAreFound_ReturnsBlurrings() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("2222222222", "cpr");

        assertEquals(2, blurrings.size());
        assertTrue(blurrings.contains(BLURRING3));
        assertTrue(blurrings.contains(BLURRING4));
    }

    @Test
    public void getActiveBlurrings_WhenMatchingExpiredBlurringIsFound_ReturnsEmptyList() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("3333333333", "cpr");

        assertEquals(0, blurrings.size());
    }

    @Test
    public void getActiveBlurrings_WhenNoMatchingBlurringIsFound_ReturnsEmptyList() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("4444444444", "cpr");

        assertEquals(0, blurrings.size());
    }

    @Test
    public void getActiveBlurrings_AfterInsert_ReturnsBlurring() {
        BlurringEntity blurringEntity = new BlurringEntity("0000000000", "cpr", "5555", "cvr", instantFrom(3000, 1, 1, 0, 0));

        blurringDao.insert(blurringEntity);
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings(blurringEntity.getPatientId(), blurringEntity.getPatientIdType());

        assertEquals(1, blurrings.size());
        assertEquals(blurringEntity, blurrings.get(0));
    }

    private static Instant instantFrom(int year, int month, int dayOfMonth, int hour, int minute) {
        return LocalDateTime.of(year, month, dayOfMonth, hour, minute).atZone(ZoneId.systemDefault()).toInstant();
    }
}