package dk.nsp.idsas.test.stepdefinitions;

import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import io.cucumber.java.en.And;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;

import dk.nsp.test.idp.CitizenIdentities;
import dk.nsp.test.idp.EmployeeIdentities;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.test.setup.TestSetup;

import static dk.nsp.idsas.test.util.Util.toXmlGregorian;
import static org.junit.Assert.*;

public class BlurringStepDefinitions {

    @Value("${idsas.service.endpoint}" + "/20230601/service")
    private String idsasServiceUrl;

    @Value("${idsas.service-lookup.endpoint}" + "/20230601/lookup")
    private String idsasServiceLookupUrl;

    @Value("${idsas.allowed.client.system.name}")
    private String clientSystemName;

    @Autowired
    private TestSetup testSetup;

    private final ServiceCaller serviceCaller = new ServiceCaller();

    private String response = "";

    private String lookupTestIdentityCVR = "98021838";

    @When("en {string} opretter en slørring for en borger med CPR {string}")
    public void enOpretterEnSlørringForEnBorgerMedCPR(String brugertype, String cpr) throws DatatypeConfigurationException {
        opretSlørring(brugertype, cpr, LocalDateTime.now().plusDays(1));
    }


    @And("der laves lookup på {string}, med clientName {string}, for at sem om der er en slørring")
    public void derLavesLookupPåMedClientNameForAtSemOmDerErEnSlørring(String cpr, String clientName) {
        try {
            lookupSlørring(cpr, clientName);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

    @Then("så fejler lookup med fejl på clientName {string}")
    public void såFejlerLookupMedFejlPåClientName(String clientName) {
        String expectedError = ErrorMessages.clientSystemNameError(clientName, clientSystemName);
        assertTrue(response.contains(expectedError));
    }

    @Then("så bliver slørringen oprettet")
    public void såBliverSlørringenOprettet() {
        assertEquals("OK", response);
    }

    @Then("så fejler oprettelsen af slørringen med forkert brugertype")
    public void såFejlerOprettelsenAfSlørringen() {
        assertTrue(response.contains(ErrorMessages.invalidUserType()));
    }

    @When("en {string} opretter en slørring for en borger med CPR {string}, som udløb i går")
    public void enOpretterEnSlørringForEnBorgerMedCPRSomUdløberIGår(String brugertype, String cpr) throws DatatypeConfigurationException {
        // Slørringen udløber i går
        opretSlørring(brugertype, cpr, LocalDateTime.now().minusDays(1));
    }

    @Then("så har {string} ikke længere aktuelle slørring over for det cvr-nummer")
    public void såHarIkkeLængereAktuelleSlørringOverForDetCvrNummer(String cpr) throws JAXBException, URISyntaxException {
        List<String> result = lookupSlørring(cpr, clientSystemName);
        assertFalse(result.contains(lookupTestIdentityCVR));
    }

    @Then("så fejler oprettelsen af slørringen med ugyldigt CPR {string}")
    public void såFejlerOprettelsenAfSlørringenMedUgyldigtCpr(String cpr) {
        assertTrue(response, response.contains("Value '" + cpr + "' is not facet-valid with respect to pattern"));
    }

    private List<String> lookupSlørring(String cpr, String clientName) throws URISyntaxException, JAXBException {
        GetBlurredOrganisationsRequestType request = new GetBlurredOrganisationsRequestType();
        PatientIdType patientId = new PatientIdType();
        BlurredOrganisationsResponseType response;

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);

        request.setPatientId(patientId);

        response = serviceCaller.callRoot(testSetup.testOrganisation98021838(clientName),
                request, SOAPOperation.GetBlurredOrganisations, new URI(idsasServiceLookupUrl),
                GetBlurredOrganisationsRequestType.class,
                BlurredOrganisationsResponseType.class);
        return response.getOrganisationIds().stream().map(OrganisationIdType::getId).collect(Collectors.toList());
    }

    private void opretSlørring(String brugertype, String cpr, LocalDateTime dateTime) throws DatatypeConfigurationException {
        CreateBlurringRequestType request = new CreateBlurringRequestType();
        PatientIdType patientId = new PatientIdType();

        // Default brugertype: "system"
        Identity identity = OrganizationIdentities.testOrganisation98021838();
        if (brugertype.equals("sundhedsfaglig")) {
            identity = EmployeeIdentities.lægeCharlesBabbage();
        } else if (brugertype.equals("borger")) {
            identity = CitizenIdentities.adaLovelace("https://fmk");
        }

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);
        request.setPatientId(patientId);
        request.setEndDateTime(toXmlGregorian(dateTime));
        try {
            response = serviceCaller.callRoot(identity,
                    request, SOAPOperation.CreateBlurring, new URI(idsasServiceUrl),
                    CreateBlurringRequestType.class,
                    String.class);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

}